/*
 * GNOME Hardware Browser
 *
 * Copyright (C) 2005 Alexandre Dussart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "libdetect.h"



struct hardware_info *detect_all() {
  struct cards_lst *lst = (struct cards_lst *) NULL;
  struct bus_lst *bus = (struct bus_lst *)NULL;
  
 struct cpu_info *cpu = (struct cpu_info *)NULL;
  struct memory_info *memory = (struct memory_info *)NULL;
  struct bridge_info *bridge = (struct bridge_info *)NULL;
  struct floppy_info *floppy = (struct floppy_info *)NULL;
  struct disk_info *disk = (struct disk_info *)NULL;
  struct cdrom_info *cdrom = (struct cdrom_info *)NULL;
  struct tape_info *tape = (struct tape_info *)NULL;
  struct keyboard_info *keyboard = (struct keyboard_info *)NULL;
  struct ethernet_info *ethernet = (struct ethernet_info *)NULL;
  struct wireless_info *wireless = (struct wireless_info *)NULL;
  struct monitor_info *monitor = (struct monitor_info *)NULL;
  struct modem_info *modem = (struct modem_info *)NULL;
  struct isdn_info *isdn = (struct isdn_info *)NULL;
  struct video_info *video = (struct video_info *)NULL;
  struct tvcard_info *tvcard = (struct tvcard_info *)NULL;
  struct soundcard_info *sound = (struct soundcard_info *)NULL;
  struct printer_info *printer = (struct printer_info *)NULL;
  struct scanner_info *scanner = (struct scanner_info *)NULL;
  struct webcam_info *webcam = (struct webcam_info *)NULL;
  struct mouse_info *mouse = (struct mouse_info *)NULL;
  struct joystick_info *joystick = (struct joystick_info *)NULL;
  struct others_info *others = (struct others_info*)NULL;
  struct hardware_info *hinfo, *first = (struct hardware_info *) NULL;

  struct identf_info *ideinterface = (struct identf_info *)NULL;
  struct scsintf_info *scsiinterface = (struct scsintf_info *)NULL;
  struct usbntf_info *usbinterface = (struct usbntf_info *)NULL;

  /* Initialize hardware device list                                  */
  lst = init_lst(PATH_ISA_LST, PATH_PCI_LST, PATH_PCMCIA_LST, PATH_USB_LST);
  bus = init_bus(lst);
  cpu = cpu_detect();
  memory = memory_detect(); 
  bridge = bridge_detect(bus);
  ideinterface = ideinterface_detect(bus);
  scsiinterface = scsiinterface_detect(bus);
  usbinterface = usbinterface_detect(bus);
  disk = disk_detect(bus);
  floppy = floppy_detect(bus);
  cdrom = cdrom_detect(bus);  
  tape = tape_detect(bus);
  scanner = scanner_detect(bus);
  webcam = webcam_detect(bus); 
  video = video_detect(bus);
  ethernet = ethernet_detect(bus);
  sound = soundcard_detect(bus);
  tvcard = tvcard_detect(bus); 
  keyboard= keyboard_detect();
  mouse = mouse_detect(bus);
  joystick = joystick_detect(bus);
  modem = modem_detect(bus);
  isdn = isdn_detect(bus);
  monitor = monitor_detect();
  printer = printer_detect(bus);
  others = others_detect(bus);


/*Keep the order...*/
	   if(memory) {
          first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
	   hinfo->vendor = s_unknown;
	   hinfo->model = s_unknown;
	   hinfo->total = memory->total;
	   hinfo->free = memory->free;
	   hinfo->buffers = memory->buffers;
	   hinfo->cached = memory->cached;
	   hinfo->swap_free = memory->swap_free;
	   hinfo->swap_total = memory->swap_total;
	   hinfo->type = MEMORY;
	   }



	  for(;cpu;cpu = cpu->next) {
	   if(!first){
        	first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
	      }else{
	        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
	        hinfo = hinfo->next;
	      }/*endif*/
	   hinfo->next = (struct hardware_info *)NULL;
	   hinfo->vendor = cpu->vendor;
	   hinfo->model = cpu->model;
	   hinfo->bogomips = cpu->bogomips;
	   hinfo->bugs = cpu->bugs;
	   hinfo->freq = cpu->freq;
	   hinfo->cache = cpu->cache;
	   hinfo->options = cpu->options;   
	   hinfo->type = CPU; 	
	  }


  for(;floppy;floppy = floppy->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = floppy->vendor;
   hinfo->model = floppy->model;
   hinfo->modulename = floppy->module;
   hinfo->bus = floppy->bus;
   hinfo->type = FLOPPY;
	}



  for(;disk;disk = disk->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = disk->vendor;
   hinfo->model = disk->model;
   hinfo->device = disk->device;
   hinfo->path = disk->path;
   hinfo->bus = disk->bus;
   hinfo-> cylinders = disk->cylinders;
   hinfo->heads = disk->heads;
   hinfo->sectors = disk->sectors;
   hinfo->host = disk->host;
   hinfo->did = disk->did;
   hinfo->size = disk->size;
   hinfo->type = DISK;
  }


  for(;cdrom;cdrom = cdrom->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = cdrom->vendor;
   hinfo->model = cdrom->model;
   hinfo->type = CDROM;
   hinfo->modulename = cdrom->module;
   hinfo->bus = cdrom->bus;
  }


  for(;tape;tape = tape->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = tape->vendor;
   hinfo->model = tape->model;
   hinfo->modulename = tape->module;
   hinfo->bus = tape->bus;
   hinfo->type = TAPE;
   }


  for(;ethernet;ethernet = ethernet->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = ethernet->vendor;
   hinfo->model = ethernet->model;
   hinfo->modulename = ethernet->module;
   hinfo->bus = ethernet->bus;
   hinfo->type = ETHERNETCARD;
   }


  if(monitor) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = monitor->vendor; 
   hinfo->model = monitor->model;
   hinfo->horizontal_min=monitor->horizontal_min;
   hinfo->horizontal_max=monitor->horizontal_max;
   hinfo->vertical_min=monitor->vertical_min;
   hinfo->vertical_max=monitor->vertical_max;
   hinfo->type = MONITOR;
   } 


 for(;modem;modem = modem->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = modem->vendor;
   hinfo->model = modem->model;
   hinfo->device = modem->device;
   hinfo->modulename = modem->module;
   hinfo->bus = modem->bus;
   hinfo->speed = modem->speed;
   hinfo->type = MODEM;
   }


  for(;video;video = video->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = video->vendor;
   hinfo->model = video->model;
   hinfo->server = video->server;
   hinfo->bus = video->bus;
   hinfo->memory;
   hinfo->type = VIDEOCARD;
  }




  for(;tvcard;tvcard = tvcard->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = tvcard->vendor;
   hinfo->model = tvcard->model;
   hinfo->modulename = tvcard->module;
   hinfo->bus = tvcard->bus;
   hinfo->type = TVCARD;
   }




  for(;sound;sound = sound->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = sound->vendor;
   hinfo->model = sound->model;
   hinfo->modulename = sound->module;
   hinfo->bus = sound->bus;
   hinfo->options = sound->options;
   hinfo->type = SOUNDCARD;
  }


  for(;printer;printer = printer->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = printer->vendor;
   hinfo->model = printer->model;
   hinfo->device = printer->device;
   hinfo->modulename = printer->module;
   hinfo->bus = printer->bus;
   hinfo->type = PRINTER;
   }



  for(;scanner;scanner = scanner->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = scanner->vendor;
   hinfo->model = scanner->model;
   hinfo->modulename = scanner->module;
   hinfo->bus = video->bus;
   hinfo->type = SCANNER;
  }


  for(;mouse;mouse = mouse->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = mouse->vendor;
   hinfo->model = mouse->model;
   hinfo->device = mouse->device;
   hinfo->modulename = mouse->module;
   hinfo->bus = mouse->bus;
   hinfo->type = MOUSE;
   }



 for(;keyboard;keyboard = keyboard->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = keyboard->vendor;
   hinfo->model = keyboard->model;
   hinfo->device = keyboard->device;
   hinfo->modulename = keyboard->module;
   hinfo->bus = keyboard->bus;
   hinfo->type = KEYBOARD;
   }


   for(;others;others = others->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = others->vendor;
   hinfo->model = others->model;
   hinfo->modulename = others->module;
   hinfo->bus = bridge->bus;
   hinfo->type = OTHERS;
  }




  for(;ideinterface;ideinterface = ideinterface->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = ideinterface->vendor;
   hinfo->model = ideinterface->model;
   hinfo->modulename = ideinterface->module;
   hinfo->bus = bridge->bus;
   hinfo->type = IDEINTERFACE;
  }


  for(;scsiinterface;scsiinterface = scsiinterface->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = scsiinterface->vendor;
   hinfo->model = scsiinterface->model;
   hinfo->modulename = scsiinterface->module;
   hinfo->bus = bridge->bus;
   hinfo->type = SCSIINTERFACE;
  }



  for(;usbinterface;usbinterface = usbinterface->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = usbinterface->vendor;
   hinfo->model = usbinterface->model;
   hinfo->modulename = usbinterface->module;
   hinfo->bus = bridge->bus;
   hinfo->type = USBINTERFACE;
   }


  for(;joystick;joystick = joystick->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = joystick->vendor;
   hinfo->model = joystick->model;
   hinfo->device = joystick->device;
   hinfo->modulename = joystick->module;
   hinfo->bus = joystick->bus;
   hinfo->type = JOYSTICK;
   }

  for(;isdn;isdn = isdn->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = isdn->vendor;
   hinfo->model = isdn->model;
   hinfo->modulename = isdn->module;
   hinfo->bus = isdn->bus;
   hinfo->type = ISDN;
   }



  for(;webcam;webcam = webcam->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = webcam->vendor;
   hinfo->model = webcam->model;
   hinfo->modulename = webcam->module;
   hinfo->bus = video->bus;
   hinfo->type = WEBCAM;
  }
 


  for(; wireless; wireless = wireless->next){
 if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = wireless->vendor;
   hinfo->model = wireless->model;
   hinfo->modulename = hinfo->modulename;
   hinfo->bus = bridge->bus;
   hinfo->type = WIRELESSCARD;
  }/*next wireless*/

  for(;bridge;bridge = bridge->next) {
   if(!first){
        first = hinfo = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
      }else{
        hinfo->next = (struct hardware_info *)
                                    my_malloc(sizeof(struct hardware_info));
        hinfo = hinfo->next;
      }/*endif*/
   hinfo->next = (struct hardware_info *)NULL;
   hinfo->vendor = bridge->vendor;
   hinfo->model = bridge->model;
   hinfo->modulename= bridge->module;
   hinfo->bus = bridge->bus;
   hinfo->type = BRIDGE;
  }

return first;
}
