/*
 * GNOME Hardware Browser
 *
 * Copyright (C) 2005 Alexandre Dussart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "libdetect.h"
#include "support.h"
#include "callbacks.h"

enum
{
  COL_ICON = 0,
  COL_TYPE,	
  COL_NAME,
  COL_DEVICE,
  NUM_COLS
};

static GnomeUIInfo file_menu[] = {
  GNOMEUIINFO_MENU_EXIT_ITEM(gtk_main_quit, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo options_menu[] = {
  GNOMEUIINFO_MENU_PREFERENCES_ITEM(NULL, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[] = {
  GNOMEUIINFO_MENU_ABOUT_ITEM(about_callback, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar[] = {
  GNOMEUIINFO_MENU_FILE_TREE(file_menu),
  GNOMEUIINFO_MENU_SETTINGS_TREE(options_menu),
  GNOMEUIINFO_MENU_HELP_TREE(help_menu),
  GNOMEUIINFO_END
};

gint select_callback(GtkTreeSelection *selection, gpointer data) {	
   GtkTreeIter iter;
   GtkTreeModel *model;
   struct hardware_info *hinfo = (struct hardware_info *)NULL;
 
   if(gtk_tree_selection_get_selected (selection, &model, &iter)) {

     gtk_tree_model_get (model, &iter, COL_DEVICE, &hinfo, -1);
     if(hinfo->vendor) 
    insert_text(hinfo); 	
    }

return 0;
}

GtkWidget *create_browser(void)
{
DeviceType device_type[] = {
  UNKNOWN_DEVICE,
  MEMORY,
  CPU,
  FLOPPY,
  DISK,
  CDROM,
  TAPE,
  ETHERNETCARD,
  MONITOR,
  MODEM,
  VIDEOCARD,
  TVCARD,
  SOUNDCARD,
  PRINTER,
  SCANNER,
  MOUSE,
  KEYBOARD,
  OTHERS,
  IDEINTERFACE,
  SCSIINTERFACE,
  USBINTERFACE,
  JOYSTICK,
  ISDN,
  WEBCAM,
  WIRELESSCARD,
  BRIDGE,	
};


  int i;

  struct hardware_info *hinfo = (struct hardware_info *)NULL;

  gchar buf[512];
  gchar pix_path[512];	

  GtkWidget *browser;
  GtkWidget *bonobodock;
  GtkWidget *hpaned;
  GtkWidget *scrolledwindow1;
  GtkWidget *scrolledwindow2;
  GtkWidget *appbar;
  GtkWidget *view;
  GtkWidget *frame;
  GtkWidget *label;
  GtkTooltips *tooltips;
  GtkTreeModel      *model;
  GtkTreeViewColumn *col;
  GtkCellRenderer   *renderer;
  GtkTreeStore  *store;
  GtkTreeIter   toplevel, child;
  GdkPixbuf     *pix;
  GError        *error = NULL;
  GtkTreeSelection *selection;


  browser=gnome_app_new("GNOME Hardware Browser 0.6", _("GNOME Harwdare Browser 0.6"));
  gtk_window_set_default_size(GTK_WINDOW(browser), 700, 400);
  gtk_window_set_position(GTK_WINDOW(browser), GTK_WIN_POS_CENTER);

  g_signal_connect(G_OBJECT(browser), "delete_event", gtk_main_quit, NULL);
  g_signal_connect(G_OBJECT(browser), "destroy_event", gtk_main_quit, NULL);

  bonobodock=GNOME_APP(browser)->dock;

  gnome_app_create_menus(GNOME_APP(browser), menubar);

  hpaned=gtk_hpaned_new ();
  gnome_app_set_contents(GNOME_APP(browser), hpaned);

  frame = gtk_frame_new (NULL);
  gtk_paned_pack1 (GTK_PANED (hpaned), frame, FALSE, TRUE);

  label = gtk_label_new (_("<b>Devices Panel</b>"));
  gtk_frame_set_label_widget (GTK_FRAME (frame), label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);

  scrolledwindow2=gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_usize(scrolledwindow2, 350, 400);
  gtk_container_add (GTK_CONTAINER (frame), scrolledwindow2);

  store = gtk_tree_store_new(4, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);

  hinfo = (struct hardware_info *)detect_all();

 
  for(i = 1; i != NUM_DEVICES; i++) {

  	sprintf(pix_path, "%s/share/pixmaps/ghb/%s.png", PREFIX, device2str(device_type[i]));	

	  pix = gdk_pixbuf_new_from_file(pix_path, &error);
	  if (error) {
	    g_warning ("%s\n", error->message);
	    g_error_free(error);
	    error = NULL;
	  }

	if (hinfo->type == MEMORY){
	sprintf(buf, "%d Kb/%d Kb", hinfo->free, hinfo->total);
	} else {
	sprintf(buf,"%s %s", hinfo->vendor, hinfo->model);
	}


	if(hinfo->type == device_type[i]) {
  	gtk_tree_store_append(store, &toplevel, NULL);
	gtk_tree_store_set(store, &toplevel,
				COL_TYPE, device2text(device_type[i]),	
        		        COL_ICON, pix,
				COL_NAME, buf,
				COL_DEVICE, hinfo,	
        			-1);
	


		for(;hinfo->type == device_type[i] && hinfo->next; hinfo = hinfo->next) {

			if (hinfo->type == MEMORY){
			sprintf(buf, "%d Kb/%d Kb", hinfo->free, hinfo->total);
			} else {
			sprintf(buf,"%s %s", hinfo->vendor, hinfo->model);
			}

		gtk_tree_store_append(store, &child, &toplevel);
		gtk_tree_store_set(store, &child,
		     COL_TYPE, device2text(hinfo->type),	
                     COL_ICON, pix,
		     COL_NAME, buf,
		     COL_DEVICE, hinfo,	
                     -1);
		}
	}
 }

  free(hinfo);

  model = GTK_TREE_MODEL(store);
  view = gtk_tree_view_new_with_model(model);
  col = gtk_tree_view_column_new();
  gtk_tree_view_column_set_title(col, _("Device"));
  renderer = gtk_cell_renderer_pixbuf_new();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
                                               -1,
                                               "Icon",
                                               renderer,
                                               "pixbuf", COL_ICON,
                                               NULL);



  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
                                               -1,
                                               "Type",
                                               renderer,
                                               "text", COL_TYPE,
                                               NULL);

  col = gtk_tree_view_column_new();

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
                                               -1,
                                               "Name",
                                               renderer,
                                               "text", COL_NAME,
                                               NULL);

  g_signal_connect (G_OBJECT (gtk_tree_view_get_selection(GTK_TREE_VIEW(view))), 				"changed",
		      G_CALLBACK (select_callback), NULL);

  gtk_container_add(GTK_CONTAINER(scrolledwindow2), view);

  frame = gtk_frame_new (NULL);
  gtk_paned_pack2 (GTK_PANED (hpaned), frame, FALSE, TRUE);

  label = gtk_label_new (_("<b>Informations panel</b>"));
  gtk_frame_set_label_widget (GTK_FRAME (frame), label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);

  textview=gtk_text_view_new();
  gtk_text_view_set_editable(GTK_TEXT_VIEW(textview), FALSE);
  gtk_container_add(GTK_CONTAINER(frame), textview);

  appbar=gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gnome_app_set_statusbar(GNOME_APP(browser), appbar);

  gnome_app_install_menu_hints (GNOME_APP(browser), menubar);

  return browser;
}

int main (int argc, char *argv[])
{
  GtkWidget *browser;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE,
                      argc, argv,
                      GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
                      NULL);

  app_splash_init(""); 

  browser=create_browser();
  gtk_widget_show_all(browser);

  app_splash_done();

  gtk_main();

  return 0;
}

